﻿using System;
namespace Persistence {

  #region Serialization Tokens

  public interface ISerializationToken {
  }
  public class BeginToken : ISerializationToken {
    public DataMemberAccessor Accessor;
    public int Id;
    public BeginToken(DataMemberAccessor accessor, int id) {
      this.Accessor = accessor;
      this.Id = id;
    }
  }
  public class EndToken : ISerializationToken {
    public DataMemberAccessor Accessor;
    public int Id;
    public EndToken(DataMemberAccessor accessor, int id) {
      this.Accessor = accessor;
      this.Id = id;
    }
  }
  public class PrimitiveToken : ISerializationToken {
    public DataMemberAccessor accessor;
    public object Data;
    public PrimitiveToken(DataMemberAccessor accessor, object data) {
      this.accessor = accessor;
      this.Data = data;
    }
  }
  public class ReferenceToken : ISerializationToken {
    public string Name;
    public int Id;
    public ReferenceToken(string name, int id) {
      this.Name = name;
      this.Id = id;
    }
  }
  public class NullReferenceToken : ISerializationToken {
    public string Name;
    public NullReferenceToken(string name) {
      this.Name = name;
    }
  }
  #endregion

  #region DeSerialization Tokens

  public interface IParseToken { }

  public class CompositeStart : IParseToken {
    public string Name;
    public Type Type;
    public int Id;
    public CompositeStart(string name, Type type, int id) {
      this.Name = name;
      this.Type = type;
      this.Id = id;
    }
  }
  public class CompositeEnd : IParseToken {
    public string Name;
    public Type Type;
    public int Id;
    public CompositeEnd(string name, Type type, int id) {
      this.Name = name;
      this.Type = type;
      this.Id = id;
    }
  }
  public class Primitive : IParseToken {
    public string Name;
    public Type Type;
    public string SerializedValue;
    public Primitive(string name, Type type, string serilaizedValue) {
      this.Name = name;
      this.Type = type;
      this.SerializedValue = serilaizedValue;
    }
  }
  public class Reference : IParseToken {
    public string Name;
    public int Id;
    public Reference(string name, int id) {
      this.Name = name;
      this.Id = id;
    }
  }
  public class Null : IParseToken {
    public string Name;
    public Null(string name) {
      this.Name = name;
    }
  }
  #endregion

}