﻿using System.Collections.Generic;
using System;
namespace Persistence {
  public class XmlFormatter {

    delegate string Formatter(ISerializationToken token);

    private readonly Dictionary<Type, Formatter> formatters;
    private int depth;

    public XmlFormatter() {
      formatters = new Dictionary<Type, Formatter>{
                       {typeof (BeginToken), FormatBegin},
                       {typeof (EndToken), FormatEnd},
                       {typeof (PrimitiveToken), FormatData},
                       {typeof (ReferenceToken), FormatReference},
                       {typeof (NullReferenceToken), FormatNullReference}
                     };
      depth = 0;
    }

    public string Format(ISerializationToken token) {
      return formatters[token.GetType()](token);
    }

    private string Prefix {
      get { return new string(' ', depth * 2); }
    }

    private string FormatBegin(ISerializationToken token) {
      BeginToken beginToken = (BeginToken)token;
      string result =
        String.Format("{0}<COMPOSITE name=\"{1}\" type=\"{2}\" id=\"{3}\">\n",
          Prefix, beginToken.Accessor.Name, beginToken.Accessor.Get().GetType(), beginToken.Id);
      depth += 1;
      return result;
    }

    private string FormatEnd(ISerializationToken token) {
      depth -= 1;
      return Prefix + "</COMPOSITE>\n";
    }

    private string FormatData(ISerializationToken token) {
      PrimitiveToken dataToken = (PrimitiveToken)token;
      return String.Format("{0}<PRIMITIVE name=\"{1}\" type=\"{2}\">{3}</PRIMITIVE>\n",
        Prefix, dataToken.accessor.Name, dataToken.accessor.Get().GetType(), dataToken.Data);
    }

    private string FormatReference(ISerializationToken token) {
      ReferenceToken refToken = (ReferenceToken)token;
      return String.Format("{0}<REFERENCE name=\"{1}\" ref=\"{2}\"/>\n",
        Prefix, refToken.Name, refToken.Id);
    }

    private string FormatNullReference(ISerializationToken token) {
      NullReferenceToken nullRefToken = (NullReferenceToken)token;
      return String.Format("{0}<NULL name=\"{1}\"/>\n",
        Prefix, nullRefToken.Name);
    }
  }
}